/*
-02/22/2006: Petre M.
  -this file contains functions used to manage the power supply of ADE7169Fxx
*/

#include "ADE7169F16.h"
#include "extern_declarations.h"
#include "EEPROM_locations.h"
#include "intrins.h"


//this routine initializes the SFRs correspondent to power management
void Setup_Power(void) {
  //bit2=0, i.e.no external reference connected to REFIN pin
  //bit1,0=01=RX pin enabled with wake-up disabled
  //      When UART communication is developed, these bits must be set!!!
  //Until then, set these pins as GPIO!!!
  PERIPH = (PERIPH & 0xF8)|Bit0;

  return;
}

//this routine initializes the SFRs correspondent to power management in PSM0
void Setup_Power_PSM0(void) {

  //clear eventual flags in IPSMF
  IPSMF = 0x00;

  /*000000=reserved
          00=battery switchover enabled on Low VDD only
  */
  BATPR = 0x00;

  //background battery measurements are enabled every 1 minute
  STRBPER=STRBPER | 0x0c;

  BATVTH = Battery_Low;//we need to introduce here a real threshold!!!!


  /*0=no PSM interrupt when VSW passes from VBAT to VDD because this is something
      PSM1 and PSM2 modes have to deal with
     x=reserved (0 by default)
      0=no PSM interrupt for SAG event
       0=reserved
        0=no PSM interrupt for VSW related events
         0=disable PSM interrupt for VBAT monitoring
          0=no PSM interrupt when VSW switches from VDD to VBAT
           0=disable PSM interrupt for VDCIN<1.2V
  */
  IPSME = 0x00;
//disable PSM interrupt in IEIP2
  IEIP2 &= (~EPSM);

  return;
}

//If a backup command has been received, then the state of the energy meter is stored
//into the EEPROM. We test IPSMF_bit.FVDC 3 times and we command the backup only if
//it is set to 1 for 3 times in a raw. Also, we want to execute it only one time,
//so we use Bit0 of Command_Status[0] to manage this.
void Backup_State(void) {

  //if a command to execute a backup has been given because VDCIN<1.2V
  if (FVDC) {
    FVDC = 0;
    //after every reading I need to wait 1usec because the EFT pulses are 1usec long
    _nop_();
    _nop_();
    _nop_();
    _nop_();
    if (FVDC) {
      FVDC = 0;
      _nop_();
      _nop_();
      _nop_();
      _nop_();
      if (FVDC) {
        FVDC = 0;
        _nop_();
        _nop_();
        _nop_();
        _nop_();
        if (!(Command_Status[0]&Bit0)) {

          //erase an eventual ALARM based command
          //clear ALARM flag
          TIMECON &= (~ALARM);

          //store the energies in EEPROM
          Active_Energy_write();

          //we read this byte to ensure the write process ends before the processor enters in sleep mode
          Tx_CTRL_byte(Nr_Bytes_1, EEPROM_Fault_Days,&Fault_Days);

          //wait until the backup has finished to switch on to the battery
          //000000=reserved
          //      01=battery switchover enabled on Low VDD and VDCIN<1.2V
          BATPR = 0x01;

          Set_flag(Bit0, &Command_Status[0]);

          //refresh watchdog here because the Alarm interrupt has been
          //disabled and it can be that the watchdog is not anymore refreshed
          Refresh_Watchdog();//this function prevents watchdog from timeout
        }
      }
    }
  }

  return;
}

//this functions blinks the baterry icon of the LCD if VBAT<BATVTH with the frequency
//determined by the frequency of ALARM RTC interrupts (currently at 0.5sec period)
//The blink option of the LCD module cannot be used because we want to
//blink only the battery icon, not the entire LCD
void Battery_Alarm(void) {
  char a;

  //if IPSMF_bit.FBAT is 1 and BATTADC<BATVTH, then blink the battery LCD icon
  if (FBAT) {
    //clear the flag
    FBAT = 0;

    //this shows that the battery voltage went below the threshold
    Set_flag(Bit3, &Command_Status[0]);

    //set a new threshold BATVTH=0xff
    BATVTH = 0xff;

    //if VBAT becomes greater than a value Battery_Low2, then stop blinking.
    //Battery_Low2 != Battery_Low in order to obtain a hysteresis
    //in triggering various modes
    a = BATADC;
    if (a>Battery_Low2) {
      //clear the battery LCD icon
#ifdef GOWORLD_LCD
      LCD_buffer[4] = LCD_buffer[4] & NBit2;
#endif
#ifdef OCULAR_LCD
      LCD_buffer[8] = LCD_buffer[8] & NBit7;
#endif
      //set a new threshold BATVTH=Battery_Low
      BATVTH = Battery_Low;

      Clear_flag(NBit3, &Command_Status[0]);
    }
  }

  //blink the battery icon as long as VBAT remains under Battery_Low,
  //i.e. as long as Bit3 of Command_Status[0] remains 1
  if (Command_Status[0] & Bit3) {
    a = BATADC;
    if (a<Battery_Low) {
      //Bit2 of LCD_buffer[4] is toggled with a period equal to ALARM RTC period
#ifdef GOWORLD_LCD
      LCD_buffer[4] = LCD_buffer[4] ^ Bit2;
#endif
#ifdef OCULAR_LCD
      LCD_buffer[8] = LCD_buffer[8] ^ Bit7;
#endif
     }
  }
  return;
}


//this function executes some tasks that must be
//done before entering Deep Sleep Mode PSM2
void Prepare_for_Sleep(void) {

  SCRATCH3 = Meter_Errors[0];
//  SCRATCH4 = LCD_Command[0];


  return;
}

//this function reads SCRATCH SFRs and various data from
//EEPROM when the processor is awoken from PSM2
void Init_variables_PSM1(void) {

  Meter_Errors[0] = SCRATCH3;
//  LCD_Command[0] = SCRATCH4;

  //set LCD_Command[0] to visualize Time in Battery Mode. In this way, if the
  //meter is powered up in Battery Mode, the display shows the time
  LCD_Command[0] = 24;

  return;
}



