/*
-08/15/2006: Petre M.
  -this file contains all the functions used in UART communications

*/
#include "ADE7169F16.h"
#include "extern_declarations.h"
#include "EEPROM_locations.h"

#if 0
void Setup_UART(void) {
   /*01=Mode 1=8 bit UART, variable rate
        0=RI is set as soon as the data byte is received
         1=serial port reception enabled
          0=this bit represents the 9th data bit transmitted in Modes 2 and 3
           0=this bit represents the stop bit received in Mode 1 case
            0=Tx INT flag
             0=Rx INT flag
    */
  SCON = 0x50;

  //for 9600baud@4.096MHz, DIV=4 and SBAUDF=0xab, real freq=9570baud
  SBAUDT = 0x04;
  SBAUDF = 0xab;

  //initialize the pointer at the beginning of UART_Rx buffer
  UART_ptr = &UART_Rx[0];

  return;
}


void Manage_UART_Rx(void) {
  //if one byte has been received
  if (RI) {
    //store SBUF in the buffer indicated by UART_ptr
    *UART_ptr=SBUF;
    UART_ptr++;
    RI=0;

    //if 11 bytes have been received
    if (UART_ptr==(&UART_Rx[10]+1)) {
      //set UART_ptr at the beginning of the buffer to be ready
      //for a new message
      UART_ptr=&UART_Rx[0];

      //before excuting the command, tests must be done to ensure a proper
      //command has been received. Bytes 7,8,9,10 must always be 0 if
      //the command was aproper one
      if (UART_Rx[7] == 0) {
        if (UART_Rx[8] == 0) {
          if (UART_Rx[9] == 0) {
            if (UART_Rx[10] ==0) {
              //execute the command only if UART_Rx[6]<sizeof(Table_UART)/2
              if (UART_Rx[6]<16) {
                UART_func_ptr =  (void (*) (void))Table_UART[UART_Rx[6]];

                //execute the command
                (*UART_func_ptr)();
              }
            }
          }
        }
      }

      //send back the message
      SBUF = *UART_ptr;
      UART_ptr++;
    }
  }

  if (TI) {
    TI = 0;
    if (UART_ptr<(&UART_Rx[10]+1)) {
      SBUF = *UART_ptr;
      UART_ptr++;
    }
    else
      UART_ptr = &UART_Rx[0];
  }
  return;
}
#endif
void IRAM_read(void) {
  char idata *iram_ptr;
  char a;

  iram_ptr = (char idata *) UART_Rx[4];
  a = *iram_ptr;

  End_message_bytes(a, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

void IRAM_write(void) {
  char idata *iram_ptr;
  iram_ptr = (char idata *) UART_Rx[4];
  *iram_ptr = UART_Rx[0];

  End_message_bytes(End_Byte_7, End_Byte_8, End_Byte_9, End_Byte_10);
  return;
}

//this function reads one ADE internal register and then prepares the
//buffer that will be sent back to PC
void ADE_reg_read(void) {

  Read_ADE_SFR(&UART_Rx[7], UART_Rx[4]);

  UART_Rx[10] = 0xaa;
  return;
}

//this function writes one ADE internal register and then prepares the
//buffer that will be sent back to PC
void ADE_reg_write(void) {

  Write_ADE_SFR(UART_Rx[1], UART_Rx[0], UART_Rx[4]);

  End_message_bytes(End_Byte_7, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

void INTPR_read(void) {
  char a;
  a = INTPR;

  End_message_bytes(a, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

void INTPR_write(void) {

  INTPR = UART_Rx[0];

  End_message_bytes(End_Byte_7, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

void RTCCOMP_read(void) {
  char a;
  a = RTCCOMP;

  End_message_bytes(a, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

void RTCCOMP_write(void) {
  RTCCOMP = UART_Rx[0];

  End_message_bytes(End_Byte_7, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}


void TEMPCAL_read(void) {
  char a;
  a = TEMPCAL;

  End_message_bytes(a, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

void TEMPCAL_write(void) {
  TEMPCAL = UART_Rx[0];

  End_message_bytes(End_Byte_7, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

void VRMS_read(void) {
  char a,b,c;

  c = VRMSH;
  b = VRMSM;
  a = VRMSL;
  End_message_bytes(a, b, c, End_Byte_10);

  return;
}


void IRMS_read(void) {
  char a,b,c;

  c = IRMSH;
  b = IRMSM;
  a = IRMSL;
  End_message_bytes(a, b, c, End_Byte_10);

  return;
}

void Time_read(void) {

  Store_time((char idata *)&UART_Rx[7]);
  return;
}

//Because the function pointer that calls Time_write() does not have
//a parameter, a new function has been introduced
//For protection reasons, the function that deals with KYREG has
//to have the value Key as a parameter
void Time_write2(unsigned char Key) {

  KYREG = Key;
  HTHSEC = UART_Rx[0];

  KYREG = Key;
  SEC = UART_Rx[1];

  KYREG = Key;
  MIN = UART_Rx[2];

  KYREG = Key;
  HOUR = UART_Rx[3];

  if (HTHSEC!=UART_Rx[0]) {
    KYREG = Key;
    HTHSEC = UART_Rx[0];

    KYREG = Key;
    SEC = UART_Rx[1];

    KYREG = Key;
    MIN = UART_Rx[2];

    KYREG = Key;
    HOUR = UART_Rx[3];
  }
  return;
}


//this function writes the RTC SFRs
void Time_write(void) {

  Time_write2(0xea);

  End_message_bytes(End_Byte_7, End_Byte_8, End_Byte_9, End_Byte_10);

  return;
}

//this function reads one byte from the EEPROM
void EEPROM_read(void) {
  int idata * iram_ptr;

  //Keil compiler uses Big Endian format for 16bit words, so 
  //it needs MSB at address A and LSB at address A+1
  Temporary[0] = UART_Rx[5];
  Temporary[1] = UART_Rx[4];

  iram_ptr = (int idata *)&Temporary[0];

  Tx_CTRL_byte(Nr_Bytes_1, *iram_ptr,(unsigned char idata  *)&UART_Rx[7]);

  UART_Rx[8] = End_Byte_8;
  UART_Rx[9] = End_Byte_9;
  UART_Rx[10]= End_Byte_10;

  return;
}

//this function writes a byte into the EEPROM and then reads it back and
//sends the byte back (this in order to be sure that the write operation has ended)
void EEPROM_write(void) {
  int idata * iram_ptr;

  //Keil compiler uses Big Endian format for 16bit words, so 
  //it needs MSB at address A and LSB at address A+1
  Temporary[0] = UART_Rx[5];
  Temporary[1] = UART_Rx[4];

  iram_ptr = (int idata *)&Temporary[0];

  Tx_byte(Nr_Bytes_1, *iram_ptr, (unsigned char idata  *)&UART_Rx[0]);

  Tx_CTRL_byte(Nr_Bytes_1, *iram_ptr,(unsigned char idata  *)&UART_Rx[7]);

  UART_Rx[8] = End_Byte_8;
  UART_Rx[9] = End_Byte_9;
  UART_Rx[10]= End_Byte_10;
  return;
}


void End_message_bytes(char a, char b, char c, char d) {

  UART_Rx[7]=a;
  UART_Rx[8]=b;
  UART_Rx[9]=c;
  UART_Rx[10] = d;
  return;
}

