/*
-02/24/2006: Petre M.
  -this file contains functions that may be charaterized as utilities
*/

#include "ADE7169F16.h"
#include "extern_declarations.h"



//this function sets the watchdog period
//PRE=0, 1,2,...,7 = properly bits PRE of WDCON
//WDCON can only be modified in a sequence of 2 instructions. For this reason,
//the optimization is turned off for this function
//#pragma optimize=none
void Setup_Watchdog(char PRE) {

  //WDE=bit1=1
  //WDWR=bit0=1
  PRE = (PRE<<4) | 0x03;

  WDWR = 1;
  WDCON = PRE;

  return;
}

//this function prevents watchdog from timeout
void Refresh_Watchdog(void) {

  EA = 0;//disable all interrupts
  WDWR = 1;
  WDE = 1;
  EA = 1;//enable all interrupts

  return;
}

//this function tests if the reset has been provoked by a watchdog
//and displays error message
void Test_Watchdog(void) {
  char i, a;

  //only if the watchdog is enabled (Bit1), test WDS bit (Bit2) and see if the
  //watchdog reset happened
  if ((WDCON & 0x06)==0x06) {
      //disable all interrupts
      EA = 0;

      Disable_Watchdog();

      Setup_LCD();

      //LCD_buffer is turned completely on and blinks with 1Hz frequency
      //BLKFREQ bits of LCDCLK SFR are 0
      Blink_LCD(Blink_1Hz);

      LCDCONY |= REFRESH;  //LCD data memory is updated, so bit0=1
      a = 0x8e;
      for (i=0; i<15; i++) {
        LCDDAT = 0xff;
        LCDPTR = a;
        a--;
      }
      LCDCONY &= (~REFRESH);  //updating LCD data memory has ended, so bit0=0

      while(1);//stay here
  }
  return;
}

void Disable_Watchdog(void) {

  WDWR = 1;
  WDE = 0;


  return;
}

//this function clears the bit identified by !NBit
void Clear_flag(char NBit, char idata  *iram_ptr) {

  char a;
  //we compute this here to avoid spending to much time with interrupts disabled
  a = *iram_ptr & NBit;

  EA = 0;//disable all interrupts
    *iram_ptr = a;
  EA = 1;//enable all interrupts
  return;
}
/*
//this function clears two bits identified by !NBit1 and !NBit2
void Clear_2_flags(char NBit_1, char idata  *iram_ptr1, char NBit_2, char idata  *iram_ptr2 ) {

  char a,b;
  a = *iram_ptr1 & NBit_1;
  b = *iram_ptr2 & NBit_2;
  EA = 0;//disable all interrupts
    *iram_ptr1 = a;
    *iram_ptr2 = b;
  EA = 1;//enable all interrupts
  return;
}
*/

//this function sets a bit identified by Bit
void Set_flag(char Bit, char idata  *iram_ptr) {

  char a;

  a = *iram_ptr | Bit;
  EA = 0;//disable all interrupts
    *iram_ptr = a;
  EA = 1;//enable all interrupts
  return;
}



//this function initializes variables that have common values for PSM0 and PSM1.
//We want to avoid the compiler doing it
//every time the part is reset or is woken up from PSM2 mode
void Init_variables(void) {
  char i;

  //LCD_buffer is initialized here
  for(i=0;i<15;i++) {
    LCD_buffer[i] = 0;
  }

  Low_Priority_Command[0] = 0;

  Command_Status[0] = 0;
  I2C_Status = 0;

  //the pointer that manages the Low Priority Tasks is initialized
  LP_Task = 0x00;

	////!!!!!!!!!!!!!!!!!!!!!!!!Only in Keil!!!!!!!!!!!!!!!!!!!!!!!
	LCD_Command[0] = 0;

  return;
}

//this function initializes variables special for PSM0
void Init_variables_PSM0(void) {
  char a;

  Meter_Errors[0] = 0;

  //this part of the code refers to the project header that is placed at
  //address 0x3e00 in flash memory. The labels need to be mentioned in
  //the program, otherwise they would not be compiled
  a = Project_Name[0];
  Temporary[0] = a;
  a = Project_Vers[0];
  Temporary[1] = a;
  a = Project_Date[0];
  Temporary[2] = a;
  a = Project_Time[0];
  Temporary[3] = a;
  a = Project_Misc[0];
  Temporary[4] = a;



  return;
}


//this function loads the ADE Register with the content of ByteH,ByteM,ByteL
void Write_ADE_SFR(char ByteM, char ByteL, char Register) {

///////  MDATH = 0x00;
  MDATM = ByteM;
  MDATL = ByteL;
  MADDPT = 0x80 | Register;

  return;
}


//this function reads the ADE Register and puts the result into ByteH, ByteM, ByteL
void Read_ADE_SFR(unsigned char idata *Pointer, char Register) {
  char ByteH, ByteM, ByteL;

  MADDPT = Register;
  ByteH = MDATH;
  ByteM = MDATM;
  ByteL = MDATL;

  *Pointer = ByteL;
  *(Pointer+1) = ByteM;
  *(Pointer+2) = ByteH;

  return;
}


//this function is always the last function that is written in the
//Table_Low_Priority_Func table. It ensures that Bit0 of Low_Priority_Command[0]
void End_Low_Priority_Tasks(void) {

  //the pointer that manages the Low Priority Tasks is initialized back to 0
  LP_Task = -1;

  Clear_flag(NBit0, &Low_Priority_Command[0]);

  return;
}


//this functions disables internal pull up resistors because
//they consume power during battery mode
void Disable_Pull_Ups(void) {

//disable the the weak pull up resistors on P0.3 (CF2) and P0.2 (CF1)
//Make sure the state is HIGH before disabling
  if (!CF1) {
    CF1 = 1;
  }
  PINMAP0 |= PINMAP02;

  if (!CF2) {
    CF2 = 1;
  }
  PINMAP0 |= PINMAP03;

  return;
}

void Enable_Pull_Ups(void) {
  PINMAP0 &= ~PINMAP02;
  PINMAP0 &= ~PINMAP03;
  return;
}

